package dev.xpple.seedmapper.util.maps;

import com.google.common.collect.ImmutableMap;
import com.seedfinding.mcbiome.biome.Biome;
import com.seedfinding.mccore.block.Blocks;
import com.seedfinding.mccore.state.Dimension;
import com.seedfinding.mccore.version.MCVersion;
import net.minecraft.block.Block;

import java.util.Map;

import static com.seedfinding.mcbiome.biome.Biomes.*;
import static net.minecraft.block.Blocks.*;

/**
 * "ungenerate" different blocks to compare them with the seed.
 * @see <a href="https://minecraft.fandom.com/wiki/Random_patch">wiki/Random_patch</a>
 */
public class SimpleBlockMap {

    private static final int AIR_ID = Blocks.AIR.getId();
    private static final int STONE_ID = Blocks.STONE.getId();
    private static final int END_STONE_ID = Blocks.END_STONE.getId();
    private static final int NETHERRACK_ID = Blocks.NETHERRACK.getId();
    private static final int WATER_ID = Blocks.WATER.getId();
    private static final int LAVA_ID = Blocks.LAVA.getId();

    private static final Map<MCVersion, Map<Biome, Map<Block, Integer>>> versionedBiomeMap = ImmutableMap.<MCVersion, Map<Biome, Map<Block, Integer>>>builder()
            .put(MCVersion.v1_16_5, ImmutableMap.<Biome, Map<Block, Integer>>builder()
                    .put(OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(PLAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(AZURE_BLUET, AIR_ID)
                            .put(OXEYE_DAISY, AIR_ID)
                            .put(CORNFLOWER, AIR_ID)
                            .put(ORANGE_TULIP, AIR_ID)
                            .put(PINK_TULIP, AIR_ID)
                            .put(RED_TULIP, AIR_ID)
                            .put(WHITE_TULIP, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .put(DANDELION, AIR_ID)
                            .build()
                    ).put(DESERT, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .build()
                    ).put(MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .put(DANDELION, AIR_ID)
                            .build()
                    ).put(FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SWAMP, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(WATER, WATER_ID)
                            .put(LILY_PAD, AIR_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(CLAY, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BLUE_ORCHID, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(RIVER, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SUGAR_CANE, AIR_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .build()
                    ).put(NETHER_WASTES, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(GLOWSTONE, AIR_ID)
                            .put(SOUL_SAND, NETHERRACK_ID)
                            .put(NETHER_QUARTZ_ORE, NETHERRACK_ID)
                            .put(NETHER_GOLD_ORE, NETHERRACK_ID)
                            .put(ANCIENT_DEBRIS, NETHERRACK_ID)
                            .put(MAGMA_BLOCK, NETHERRACK_ID)
                            .put(LAVA, LAVA_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(FIRE, AIR_ID)
                            .build()
                    ).put(THE_END, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .put(OBSIDIAN, AIR_ID)
                            .put(BEDROCK, AIR_ID)
                            .build()
                    ).put(FROZEN_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(ICE, STONE_ID)
                            .put(SNOW_BLOCK, STONE_ID)
                            .put(PACKED_ICE, STONE_ID)
                            .put(BLUE_ICE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .build()
                    ).put(FROZEN_RIVER, ImmutableMap.<Block, Integer>builder()
                            .put(ICE, STONE_ID)
                            .put(WATER, WATER_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .build()
                    ).put(SNOWY_TUNDRA, ImmutableMap.<Block, Integer>builder()
                            .put(SNOW, AIR_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(ICE, STONE_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SNOWY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(SNOW, AIR_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(ICE, STONE_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(MUSHROOM_FIELDS, ImmutableMap.<Block, Integer>builder()
                            .put(MYCELIUM, STONE_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    ).put(MUSHROOM_FIELD_SHORE, ImmutableMap.<Block, Integer>builder()
                            .put(MYCELIUM, STONE_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    ).put(BEACH, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SUGAR_CANE, AIR_ID)
                            .build()
                    ).put(DESERT_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .build()
                    ).put(WOODED_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(MOUNTAIN_EDGE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)
                            .build()
                    ).put(JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .put(BAMBOO, AIR_ID)
                            .build()
                    ).put(JUNGLE_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .put(BAMBOO, AIR_ID)
                            .build()
                    ).put(JUNGLE_EDGE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .build()
                    ).put(DEEP_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(STONE_SHORE, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(STONE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .build()
                    ).put(SNOWY_BEACH, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .build()
                    ).put(BIRCH_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(BIRCH_FOREST_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(DANDELION, AIR_ID)
                            .build()
                    ).put(DARK_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DARK_OAK_LOG, AIR_ID)
                            .put(DARK_OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(SNOWY_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SNOWY_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(GIANT_TREE_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(GIANT_TREE_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(WOODED_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(SAVANNA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)

                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(SAVANNA_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(BADLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(WOODED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(OAK_LOG, STONE_ID)
                            .put(OAK_LEAVES, STONE_ID)
                            .build()
                    ).put(BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(SMALL_END_ISLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .build()
                    ).put(END_MIDLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .build()
                    ).put(END_HIGHLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .put(CHORUS_PLANT, AIR_ID)
                            .put(CHORUS_FLOWER, AIR_ID)
                            .build()
                    ).put(END_BARRENS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .build()
                    ).put(WARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(BRAIN_CORAL_BLOCK, STONE_ID)
                            .put(BRAIN_CORAL_FAN, STONE_ID)
                            .put(BUBBLE_CORAL_BLOCK, STONE_ID)
                            .put(BUBBLE_CORAL_FAN, STONE_ID)
                            .put(DEAD_BRAIN_CORAL_BLOCK, STONE_ID)
                            .put(DEAD_BRAIN_CORAL_FAN, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL_BLOCK, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL_FAN, STONE_ID)
                            .put(FIRE_CORAL_BLOCK, STONE_ID)
                            .put(FIRE_CORAL_FAN, STONE_ID)
                            .put(DEAD_FIRE_CORAL_BLOCK, STONE_ID)
                            .put(DEAD_FIRE_CORAL_FAN, STONE_ID)
                            .put(HORN_CORAL_BLOCK, STONE_ID)
                            .put(HORN_CORAL_FAN, STONE_ID)
                            .put(TUBE_CORAL_BLOCK, STONE_ID)
                            .put(TUBE_CORAL_FAN, STONE_ID)
                            .put(BRAIN_CORAL_WALL_FAN, STONE_ID)
                            .put(BRAIN_CORAL, STONE_ID)
                            .put(BUBBLE_CORAL_WALL_FAN, STONE_ID)
                            .put(BUBBLE_CORAL, STONE_ID)
                            .put(DEAD_BRAIN_CORAL_WALL_FAN, STONE_ID)
                            .put(DEAD_BRAIN_CORAL, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL_WALL_FAN, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL, STONE_ID)
                            .put(FIRE_CORAL_WALL_FAN, STONE_ID)
                            .put(FIRE_CORAL, STONE_ID)
                            .put(DEAD_FIRE_CORAL_WALL_FAN, STONE_ID)
                            .put(DEAD_FIRE_CORAL, STONE_ID)
                            .put(HORN_CORAL_WALL_FAN, STONE_ID)
                            .put(HORN_CORAL, STONE_ID)
                            .put(TUBE_CORAL_WALL_FAN, STONE_ID)
                            .put(TUBE_CORAL, STONE_ID)
                            .put(SEA_PICKLE, STONE_ID)
                            .build()
                    ).put(LUKEWARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(COLD_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(DEEP_WARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SAND, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .build()
                    ).put(DEEP_LUKEWARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SAND, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(DEEP_COLD_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(DEEP_FROZEN_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SNOW_BLOCK, STONE_ID)
                            .put(PACKED_ICE, STONE_ID)
                            .put(ICE, STONE_ID)
                            .put(BLUE_ICE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .build()
                    ).put(THE_VOID, ImmutableMap.<Block, Integer>builder()
                            .put(COBBLESTONE, STONE_ID)
                            .put(STONE, STONE_ID)
                            .build()
                    ).put(SUNFLOWER_PLAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(AZURE_BLUET, AIR_ID)
                            .put(OXEYE_DAISY, AIR_ID)
                            .put(CORNFLOWER, AIR_ID)
                            .put(ORANGE_TULIP, AIR_ID)
                            .put(PINK_TULIP, AIR_ID)
                            .put(RED_TULIP, AIR_ID)
                            .put(WHITE_TULIP, AIR_ID)
                            .put(SUNFLOWER, AIR_ID)
                            .build()
                    ).put(DESERT_LAKES, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(WATER, WATER_ID)
                            .put(SUGAR_CANE, AIR_ID)
                            .build()
                    ).put(GRAVELLY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRAVEL, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .build()
                    ).put(FLOWER_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(ALLIUM, AIR_ID)
                            .build()
                    ).put(TAIGA_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SWAMP_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(WATER, WATER_ID)
                            .put(LILY_PAD, AIR_ID)
                            .put(CLAY, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BLUE_ORCHID, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    ).put(ICE_SPIKES, ImmutableMap.<Block, Integer>builder()
                            .put(PACKED_ICE, AIR_ID)
                            .put(SNOW_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(DIRT, STONE_ID)
                            .put(ICE, STONE_ID)
                            .build()
                    ).put(MODIFIED_JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .put(BAMBOO, AIR_ID)
                            .build()
                    ).put(MODIFIED_JUNGLE_EDGE,  ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .build()
                    ).put(TALL_BIRCH_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .build()
                    ).put(TALL_BIRCH_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .build()
                    ).put(DARK_FOREST_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DARK_OAK_LOG, AIR_ID)
                            .put(DARK_OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .build()
                    ).put(SNOWY_TAIGA_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(GIANT_SPRUCE_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(GIANT_SPRUCE_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(MODIFIED_GRAVELLY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRAVEL, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .build()
                    ).put(SHATTERED_SAVANNA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(STONE, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(SHATTERED_SAVANNA_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(STONE, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .build()
                    ).put(ERODED_BADLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(MODIFIED_WOODED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(OAK_LOG, STONE_ID)
                            .put(OAK_LEAVES, STONE_ID)
                            .build()
                    ).put(MODIFIED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(BAMBOO_JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(BAMBOO, AIR_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .build()
                    ).put(BAMBOO_JUNGLE_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(BAMBOO, AIR_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .build()
                    ).put(SOUL_SAND_VALLEY, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(LAVA, LAVA_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(SOUL_SAND, NETHERRACK_ID)
                            .put(SOUL_SOIL, NETHERRACK_ID)
                            .put(BASALT, NETHERRACK_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(BONE_BLOCK, NETHERRACK_ID)
                            .put(GLOWSTONE, AIR_ID)
                            .put(CRIMSON_ROOTS, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SOUL_FIRE, AIR_ID)
                            .build()
                    ).put(CRIMSON_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(CRIMSON_NYLIUM, NETHERRACK_ID)
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(CRIMSON_STEM, AIR_ID)
                            .put(NETHER_WART_BLOCK, AIR_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(SHROOMLIGHT, AIR_ID)
                            .put(CRIMSON_ROOTS, AIR_ID)
                            .put(CRIMSON_FUNGUS, AIR_ID)
                            .put(WARPED_FUNGUS, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(WEEPING_VINES, AIR_ID)
                            .put(WEEPING_VINES_PLANT, AIR_ID)
                            .build()
                    ).put(WARPED_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(WARPED_NYLIUM, NETHERRACK_ID)
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(WARPED_STEM, AIR_ID)
                            .put(WARPED_WART_BLOCK, AIR_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(SHROOMLIGHT, AIR_ID)
                            .put(WARPED_ROOTS, AIR_ID)
                            .put(WARPED_FUNGUS, AIR_ID)
                            .put(CRIMSON_FUNGUS, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(NETHER_SPROUTS, AIR_ID)
                            .put(TWISTING_VINES, AIR_ID)
                            .build()
                    ).put(BASALT_DELTAS, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(BASALT, LAVA_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(MAGMA_BLOCK, NETHERRACK_ID)
                            .put(LAVA, LAVA_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    )
                    .build()
            ).put(MCVersion.v1_17, ImmutableMap.<Biome, Map<Block, Integer>>builder()
                    .put(OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(PLAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(AZURE_BLUET, AIR_ID)
                            .put(OXEYE_DAISY, AIR_ID)
                            .put(CORNFLOWER, AIR_ID)
                            .put(ORANGE_TULIP, AIR_ID)
                            .put(PINK_TULIP, AIR_ID)
                            .put(RED_TULIP, AIR_ID)
                            .put(WHITE_TULIP, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .put(DANDELION, AIR_ID)
                            .build()
                    ).put(DESERT, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .build()
                    ).put(MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .put(DANDELION, AIR_ID)
                            .build()
                    ).put(FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SWAMP, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(WATER, WATER_ID)
                            .put(LILY_PAD, AIR_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(CLAY, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BLUE_ORCHID, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(RIVER, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SUGAR_CANE, AIR_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .build()
                    ).put(NETHER_WASTES, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(GLOWSTONE, AIR_ID)
                            .put(SOUL_SAND, NETHERRACK_ID)
                            .put(NETHER_QUARTZ_ORE, NETHERRACK_ID)
                            .put(NETHER_GOLD_ORE, NETHERRACK_ID)
                            .put(ANCIENT_DEBRIS, NETHERRACK_ID)
                            .put(MAGMA_BLOCK, NETHERRACK_ID)
                            .put(LAVA, LAVA_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(FIRE, AIR_ID)
                            .build()
                    ).put(THE_END, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .put(OBSIDIAN, AIR_ID)
                            .put(BEDROCK, AIR_ID)
                            .build()
                    ).put(FROZEN_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(ICE, STONE_ID)
                            .put(SNOW_BLOCK, STONE_ID)
                            .put(PACKED_ICE, STONE_ID)
                            .put(BLUE_ICE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .build()
                    ).put(FROZEN_RIVER, ImmutableMap.<Block, Integer>builder()
                            .put(ICE, STONE_ID)
                            .put(WATER, WATER_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .build()
                    ).put(SNOWY_TUNDRA, ImmutableMap.<Block, Integer>builder()
                            .put(SNOW, AIR_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(ICE, STONE_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SNOWY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(SNOW, AIR_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(ICE, STONE_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(MUSHROOM_FIELDS, ImmutableMap.<Block, Integer>builder()
                            .put(MYCELIUM, STONE_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    ).put(MUSHROOM_FIELD_SHORE, ImmutableMap.<Block, Integer>builder()
                            .put(MYCELIUM, STONE_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    ).put(BEACH, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SUGAR_CANE, AIR_ID)
                            .build()
                    ).put(DESERT_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .build()
                    ).put(WOODED_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(MOUNTAIN_EDGE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)
                            .build()
                    ).put(JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .put(BAMBOO, AIR_ID)
                            .build()
                    ).put(JUNGLE_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .put(BAMBOO, AIR_ID)
                            .build()
                    ).put(JUNGLE_EDGE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .build()
                    ).put(DEEP_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(STONE_SHORE, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(STONE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .build()
                    ).put(SNOWY_BEACH, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .build()
                    ).put(BIRCH_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(BIRCH_FOREST_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(DANDELION, AIR_ID)
                            .build()
                    ).put(DARK_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DARK_OAK_LOG, AIR_ID)
                            .put(DARK_OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(SNOWY_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SNOWY_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(GIANT_TREE_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(GIANT_TREE_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(WOODED_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(ICE, STONE_ID)

                            .put(GRASS, AIR_ID)
                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(SAVANNA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)

                            .put(POPPY, AIR_ID)
                            .build()
                    ).put(SAVANNA_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(BADLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(WOODED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(OAK_LOG, STONE_ID)
                            .put(OAK_LEAVES, STONE_ID)
                            .build()
                    ).put(BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(SMALL_END_ISLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .build()
                    ).put(END_MIDLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .build()
                    ).put(END_HIGHLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .put(CHORUS_PLANT, AIR_ID)
                            .put(CHORUS_FLOWER, AIR_ID)
                            .build()
                    ).put(END_BARRENS, ImmutableMap.<Block, Integer>builder()
                            .put(END_STONE, END_STONE_ID)
                            .build()
                    ).put(WARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(BRAIN_CORAL_BLOCK, STONE_ID)
                            .put(BRAIN_CORAL_FAN, STONE_ID)
                            .put(BUBBLE_CORAL_BLOCK, STONE_ID)
                            .put(BUBBLE_CORAL_FAN, STONE_ID)
                            .put(DEAD_BRAIN_CORAL_BLOCK, STONE_ID)
                            .put(DEAD_BRAIN_CORAL_FAN, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL_BLOCK, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL_FAN, STONE_ID)
                            .put(FIRE_CORAL_BLOCK, STONE_ID)
                            .put(FIRE_CORAL_FAN, STONE_ID)
                            .put(DEAD_FIRE_CORAL_BLOCK, STONE_ID)
                            .put(DEAD_FIRE_CORAL_FAN, STONE_ID)
                            .put(HORN_CORAL_BLOCK, STONE_ID)
                            .put(HORN_CORAL_FAN, STONE_ID)
                            .put(TUBE_CORAL_BLOCK, STONE_ID)
                            .put(TUBE_CORAL_FAN, STONE_ID)
                            .put(BRAIN_CORAL_WALL_FAN, STONE_ID)
                            .put(BRAIN_CORAL, STONE_ID)
                            .put(BUBBLE_CORAL_WALL_FAN, STONE_ID)
                            .put(BUBBLE_CORAL, STONE_ID)
                            .put(DEAD_BRAIN_CORAL_WALL_FAN, STONE_ID)
                            .put(DEAD_BRAIN_CORAL, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL_WALL_FAN, STONE_ID)
                            .put(DEAD_BUBBLE_CORAL, STONE_ID)
                            .put(FIRE_CORAL_WALL_FAN, STONE_ID)
                            .put(FIRE_CORAL, STONE_ID)
                            .put(DEAD_FIRE_CORAL_WALL_FAN, STONE_ID)
                            .put(DEAD_FIRE_CORAL, STONE_ID)
                            .put(HORN_CORAL_WALL_FAN, STONE_ID)
                            .put(HORN_CORAL, STONE_ID)
                            .put(TUBE_CORAL_WALL_FAN, STONE_ID)
                            .put(TUBE_CORAL, STONE_ID)
                            .put(SEA_PICKLE, STONE_ID)
                            .build()
                    ).put(LUKEWARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(COLD_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(SAND, STONE_ID)
                            .put(DIRT, STONE_ID)
                            .put(CLAY, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(DEEP_WARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SAND, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .build()
                    ).put(DEEP_LUKEWARM_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SAND, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(DEEP_COLD_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .put(SEAGRASS, WATER_ID)
                            .put(TALL_SEAGRASS, WATER_ID)
                            .put(KELP, WATER_ID)
                            .put(KELP_PLANT, WATER_ID)
                            .build()
                    ).put(DEEP_FROZEN_OCEAN, ImmutableMap.<Block, Integer>builder()
                            .put(WATER, WATER_ID)
                            .put(SNOW_BLOCK, STONE_ID)
                            .put(PACKED_ICE, STONE_ID)
                            .put(ICE, STONE_ID)
                            .put(BLUE_ICE, STONE_ID)
                            .put(GRAVEL, STONE_ID)
                            .put(BUBBLE_COLUMN, WATER_ID)
                            .put(MAGMA_BLOCK, STONE_ID)
                            .put(OBSIDIAN, STONE_ID)
                            .build()
                    ).put(THE_VOID, ImmutableMap.<Block, Integer>builder()
                            .put(COBBLESTONE, STONE_ID)
                            .put(STONE, STONE_ID)
                            .build()
                    ).put(SUNFLOWER_PLAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(AZURE_BLUET, AIR_ID)
                            .put(OXEYE_DAISY, AIR_ID)
                            .put(CORNFLOWER, AIR_ID)
                            .put(ORANGE_TULIP, AIR_ID)
                            .put(PINK_TULIP, AIR_ID)
                            .put(RED_TULIP, AIR_ID)
                            .put(WHITE_TULIP, AIR_ID)
                            .put(SUNFLOWER, AIR_ID)
                            .build()
                    ).put(DESERT_LAKES, ImmutableMap.<Block, Integer>builder()
                            .put(SAND, STONE_ID)
                            .put(SANDSTONE, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(WATER, WATER_ID)
                            .put(SUGAR_CANE, AIR_ID)
                            .build()
                    ).put(GRAVELLY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRAVEL, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .build()
                    ).put(FLOWER_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(ALLIUM, AIR_ID)
                            .build()
                    ).put(TAIGA_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(SWAMP_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(WATER, WATER_ID)
                            .put(LILY_PAD, AIR_ID)
                            .put(CLAY, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(BLUE_ORCHID, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    ).put(ICE_SPIKES, ImmutableMap.<Block, Integer>builder()
                            .put(PACKED_ICE, AIR_ID)
                            .put(SNOW_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(DIRT, STONE_ID)
                            .put(ICE, STONE_ID)
                            .build()
                    ).put(MODIFIED_JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .put(BAMBOO, AIR_ID)
                            .build()
                    ).put(MODIFIED_JUNGLE_EDGE,  ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(COCOA, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .build()
                    ).put(TALL_BIRCH_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .build()
                    ).put(TALL_BIRCH_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(BEE_NEST, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .build()
                    ).put(DARK_FOREST_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(DARK_OAK_LOG, AIR_ID)
                            .put(DARK_OAK_LEAVES, AIR_ID)
                            .put(BIRCH_LOG, AIR_ID)
                            .put(BIRCH_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(RED_MUSHROOM_BLOCK, AIR_ID)
                            .put(BROWN_MUSHROOM_BLOCK, AIR_ID)
                            .put(MUSHROOM_STEM, AIR_ID)
                            .put(ROSE_BUSH, AIR_ID)
                            .put(PEONY, AIR_ID)
                            .put(LILAC, AIR_ID)
                            .put(LILY_OF_THE_VALLEY, AIR_ID)
                            .build()
                    ).put(SNOWY_TAIGA_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .build()
                    ).put(GIANT_SPRUCE_TAIGA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(GIANT_SPRUCE_TAIGA_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(MOSSY_COBBLESTONE, AIR_ID)
                            .put(SPRUCE_LOG, AIR_ID)
                            .put(SPRUCE_LEAVES, AIR_ID)
                            .put(FERN, AIR_ID)
                            .put(LARGE_FERN, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SWEET_BERRY_BUSH, AIR_ID)
                            .build()
                    ).put(MODIFIED_GRAVELLY_MOUNTAINS, ImmutableMap.<Block, Integer>builder()
                            .put(GRAVEL, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(SNOW, AIR_ID)
                            .put(STONE, STONE_ID)
                            .put(INFESTED_STONE, STONE_ID)
                            .put(EMERALD_ORE, STONE_ID)
                            .put(COAL_ORE, STONE_ID)
                            .put(IRON_ORE, STONE_ID)
                            .build()
                    ).put(SHATTERED_SAVANNA, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(STONE, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .build()
                    ).put(SHATTERED_SAVANNA_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(STONE, STONE_ID)
                            .put(TALL_GRASS, AIR_ID)
                            .put(ACACIA_LOG, AIR_ID)
                            .put(ACACIA_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .build()
                    ).put(ERODED_BADLANDS, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(MODIFIED_WOODED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(COARSE_DIRT, STONE_ID)
                            .put(OAK_LOG, STONE_ID)
                            .put(OAK_LEAVES, STONE_ID)
                            .build()
                    ).put(MODIFIED_BADLANDS_PLATEAU, ImmutableMap.<Block, Integer>builder()
                            .put(RED_SAND, STONE_ID)
                            .put(TERRACOTTA, STONE_ID)
                            .put(WHITE_TERRACOTTA, STONE_ID)
                            .put(ORANGE_TERRACOTTA, STONE_ID)
                            .put(MAGENTA_TERRACOTTA, STONE_ID)
                            .put(LIGHT_BLUE_TERRACOTTA, STONE_ID)
                            .put(YELLOW_TERRACOTTA, STONE_ID)
                            .put(LIME_TERRACOTTA, STONE_ID)
                            .put(PINK_TERRACOTTA, STONE_ID)
                            .put(GRAY_TERRACOTTA, STONE_ID)
                            .put(LIGHT_GRAY_TERRACOTTA, STONE_ID)
                            .put(CYAN_TERRACOTTA, STONE_ID)
                            .put(PURPLE_TERRACOTTA, STONE_ID)
                            .put(BLUE_TERRACOTTA, STONE_ID)
                            .put(BROWN_TERRACOTTA, STONE_ID)
                            .put(GREEN_TERRACOTTA, STONE_ID)
                            .put(RED_TERRACOTTA, STONE_ID)
                            .put(BLACK_TERRACOTTA, STONE_ID)
                            .put(CACTUS, AIR_ID)
                            .put(DEAD_BUSH, AIR_ID)
                            .put(GOLD_ORE, STONE_ID)
                            .build()
                    ).put(BAMBOO_JUNGLE, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(BAMBOO, AIR_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)

                            .put(GRASS, AIR_ID)
                            .put(FERN, AIR_ID)
                            .build()
                    ).put(BAMBOO_JUNGLE_HILLS, ImmutableMap.<Block, Integer>builder()
                            .put(GRASS_BLOCK, STONE_ID)
                            .put(PODZOL, STONE_ID)
                            .put(BAMBOO, AIR_ID)
                            .put(VINE, AIR_ID)
                            .put(JUNGLE_LOG, AIR_ID)
                            .put(JUNGLE_LEAVES, AIR_ID)
                            .put(OAK_LOG, AIR_ID)
                            .put(OAK_LEAVES, AIR_ID)
                            .put(MELON, AIR_ID)
                            .build()
                    ).put(SOUL_SAND_VALLEY, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(LAVA, LAVA_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(SOUL_SAND, NETHERRACK_ID)
                            .put(SOUL_SOIL, NETHERRACK_ID)
                            .put(BASALT, NETHERRACK_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(BONE_BLOCK, NETHERRACK_ID)
                            .put(GLOWSTONE, AIR_ID)
                            .put(CRIMSON_ROOTS, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(SOUL_FIRE, AIR_ID)
                            .build()
                    ).put(CRIMSON_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(CRIMSON_NYLIUM, NETHERRACK_ID)
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(CRIMSON_STEM, AIR_ID)
                            .put(NETHER_WART_BLOCK, AIR_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(SHROOMLIGHT, AIR_ID)
                            .put(CRIMSON_ROOTS, AIR_ID)
                            .put(CRIMSON_FUNGUS, AIR_ID)
                            .put(WARPED_FUNGUS, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(WEEPING_VINES, AIR_ID)
                            .put(WEEPING_VINES_PLANT, AIR_ID)
                            .build()
                    ).put(WARPED_FOREST, ImmutableMap.<Block, Integer>builder()
                            .put(WARPED_NYLIUM, NETHERRACK_ID)
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(WARPED_STEM, AIR_ID)
                            .put(WARPED_WART_BLOCK, AIR_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(SHROOMLIGHT, AIR_ID)
                            .put(WARPED_ROOTS, AIR_ID)
                            .put(WARPED_FUNGUS, AIR_ID)
                            .put(CRIMSON_FUNGUS, AIR_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .put(NETHER_SPROUTS, AIR_ID)
                            .put(TWISTING_VINES, AIR_ID)
                            .build()
                    ).put(BASALT_DELTAS, ImmutableMap.<Block, Integer>builder()
                            .put(NETHERRACK, NETHERRACK_ID)
                            .put(BASALT, LAVA_ID)
                            .put(BLACKSTONE, NETHERRACK_ID)
                            .put(MAGMA_BLOCK, NETHERRACK_ID)
                            .put(LAVA, LAVA_ID)
                            .put(GRAVEL, NETHERRACK_ID)
                            .put(RED_MUSHROOM, AIR_ID)
                            .put(BROWN_MUSHROOM, AIR_ID)
                            .build()
                    )
                    .build()
            )
            .build();

    private static final Map<MCVersion, Map<Block, Integer>> overworld = ImmutableMap.<MCVersion, Map<Block, Integer>>builder()
            .put(MCVersion.v1_16_5, ImmutableMap.<Block, Integer>builder()
                    .put(AIR, AIR_ID)
                    .put(CAVE_AIR, STONE_ID)
                    .put(VOID_AIR, AIR_ID)
                    //.put(BEDROCK, 7)
                    .put(BEDROCK, STONE_ID)
                    .put(STONE, STONE_ID)
                    .put(DIRT, STONE_ID)
                    .put(WATER, STONE_ID)
                    .put(LAVA, STONE_ID)
                    .put(COAL_ORE, STONE_ID)
                    .put(IRON_ORE, STONE_ID)
                    .put(GOLD_ORE, STONE_ID)
                    .put(DIAMOND_ORE, STONE_ID)
                    .put(EMERALD_ORE, STONE_ID)
                    .put(LAPIS_ORE, STONE_ID)
                    .put(REDSTONE_ORE, STONE_ID)
                    .put(ANDESITE, STONE_ID)
                    .put(DIORITE, STONE_ID)
                    .put(GRANITE, STONE_ID)
                    .put(GRAVEL, STONE_ID)
                    .build()
            ).put(MCVersion.v1_17, ImmutableMap.<Block, Integer>builder()
                    .put(AIR, AIR_ID)
                    .put(CAVE_AIR, AIR_ID)
                    .put(VOID_AIR, AIR_ID)
                    //.put(BEDROCK, 7)
                    .put(BEDROCK, STONE_ID)
                    .put(STONE, STONE_ID)
                    .put(DEEPSLATE, STONE_ID)
                    .put(DIRT, STONE_ID)
                    .put(WATER, STONE_ID)
                    .put(LAVA, STONE_ID)
                    .put(COAL_ORE, STONE_ID)
                    .put(DEEPSLATE_COAL_ORE, STONE_ID)
                    .put(IRON_ORE, STONE_ID)
                    .put(DEEPSLATE_IRON_ORE, STONE_ID)
                    .put(GOLD_ORE, STONE_ID)
                    .put(DEEPSLATE_GOLD_ORE, STONE_ID)
                    .put(DIAMOND_ORE, STONE_ID)
                    .put(DEEPSLATE_DIAMOND_ORE, STONE_ID)
                    .put(EMERALD_ORE, STONE_ID)
                    .put(DEEPSLATE_EMERALD_ORE, STONE_ID)
                    .put(LAPIS_ORE, STONE_ID)
                    .put(DEEPSLATE_LAPIS_ORE, STONE_ID)
                    .put(REDSTONE_ORE, STONE_ID)
                    .put(DEEPSLATE_REDSTONE_ORE, STONE_ID)
                    .put(COPPER_ORE, STONE_ID)
                    .put(DEEPSLATE_COPPER_ORE, STONE_ID)
                    .put(ANDESITE, STONE_ID)
                    .put(DIORITE, STONE_ID)
                    .put(GRANITE, STONE_ID)
                    .put(GRAVEL, STONE_ID)
                    .put(TUFF, STONE_ID)
                    .put(DRIPSTONE_BLOCK, STONE_ID)
                    .put(POINTED_DRIPSTONE, STONE_ID)
                    .put(GLOW_LICHEN, STONE_ID)
                    .put(AMETHYST_CLUSTER, STONE_ID)
                    .put(LARGE_AMETHYST_BUD, STONE_ID)
                    .put(MEDIUM_AMETHYST_BUD, STONE_ID)
                    .put(SMALL_AMETHYST_BUD, STONE_ID)
                    .put(AMETHYST_BLOCK, STONE_ID)
                    .put(BUDDING_AMETHYST, STONE_ID)
                    .put(CALCITE, STONE_ID)
                    .put(SMOOTH_BASALT, STONE_ID)
                    .build()
            )
            .build();

    private static final Map<MCVersion, Map<Block, Integer>> nether = ImmutableMap.<MCVersion, Map<Block, Integer>>builder()
            .put(MCVersion.v1_16_5, ImmutableMap.<Block, Integer>builder()
                    .put(AIR, AIR_ID)
                    .put(CAVE_AIR, NETHERRACK_ID)
                    .put(VOID_AIR, AIR_ID)
                    //.put(BEDROCK, 7)
                    .put(BEDROCK, NETHERRACK_ID)
                    .put(NETHERRACK, NETHERRACK_ID)
                    .put(LAVA, LAVA_ID)
                    .put(NETHER_QUARTZ_ORE, NETHERRACK_ID)
                    .put(NETHER_GOLD_ORE, NETHERRACK_ID)
                    .put(BLACKSTONE, NETHERRACK_ID)
                    .put(BASALT, NETHERRACK_ID)
                    .put(ANCIENT_DEBRIS, NETHERRACK_ID)
                    .build()
            ).put(MCVersion.v1_17, ImmutableMap.<Block, Integer>builder()
                    .put(AIR, AIR_ID)
                    .put(CAVE_AIR, NETHERRACK_ID)
                    .put(VOID_AIR, AIR_ID)
                    //.put(BEDROCK, 7)
                    .put(BEDROCK, NETHERRACK_ID)
                    .put(NETHERRACK, NETHERRACK_ID)
                    .put(LAVA, LAVA_ID)
                    .put(NETHER_QUARTZ_ORE, NETHERRACK_ID)
                    .put(NETHER_GOLD_ORE, NETHERRACK_ID)
                    .put(BLACKSTONE, NETHERRACK_ID)
                    .put(BASALT, NETHERRACK_ID)
                    .put(ANCIENT_DEBRIS, NETHERRACK_ID)
                    .build()
            )
            .build();

    private static final Map<MCVersion, Map<Block, Integer>> end = ImmutableMap.<MCVersion, Map<Block, Integer>>builder()
            .put(MCVersion.v1_16_5, ImmutableMap.<Block, Integer>builder()
                    .put(AIR, AIR_ID)
                    .put(CAVE_AIR, END_STONE_ID)
                    .put(VOID_AIR, AIR_ID)
                    .put(END_STONE, END_STONE_ID)
                    .build()
            ).put(MCVersion.v1_17, ImmutableMap.<Block, Integer>builder()
                    .put(AIR, AIR_ID)
                    .put(CAVE_AIR, END_STONE_ID)
                    .put(VOID_AIR, AIR_ID)
                    .put(END_STONE, END_STONE_ID)
                    .build()
            )
            .build();

    private final MCVersion mcVersion;
    private Map<Block, Integer> map;
    private final Dimension dimension;

    public SimpleBlockMap(Dimension dimension) {
        this(dimension, PLAINS);
    }

    public SimpleBlockMap(Dimension dimension, Biome biome) {
        this(MCVersion.v1_17, dimension, biome);
    }

    public SimpleBlockMap(MCVersion mcVersion, Dimension dimension, Biome biome) {
        this.mcVersion = mcVersion;
        this.map = versionedBiomeMap.getOrDefault(this.mcVersion, versionedBiomeMap.get(MCVersion.v1_17)).get(biome);
        this.dimension = dimension;
    }

    public void setBiome(Biome biome) {
        this.map = versionedBiomeMap.getOrDefault(this.mcVersion, versionedBiomeMap.get(MCVersion.v1_17)).get(biome);
    }

    public int get(Block block) {
        Integer biomeBlock = this.map.get(block);
        if (biomeBlock == null) {
            if (this.dimension == Dimension.OVERWORLD) {
                return overworld.getOrDefault(this.mcVersion, overworld.get(MCVersion.v1_17)).getOrDefault(block, -1);
            } else if (this.dimension == Dimension.NETHER) {
                return nether.getOrDefault(this.mcVersion, nether.get(MCVersion.v1_17)).getOrDefault(block, -1);
            } else {
                return end.getOrDefault(this.mcVersion, end.get(MCVersion.v1_17)).getOrDefault(block, -1);
            }
        }
        return biomeBlock;
    }
}
